<?php
/**
 * @file
 * Hooks for Cforge distribution profile.
 *
 * @todo ensure all lists show only active users with role RID_TRADER
 */

use Drupal\aggregator\Entity\Feed;
use Drupal\user\UserInterface;
use Drupal\user\Entity\User;
use Drupal\user\Entity\Role;
use Drupal\Core\Render\Element;
use Drupal\Core\Extension\Extension;
use Drupal\Core\Entity\Entity\EntityFormDisplay;
use Drupal\Core\Url;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Batch\BatchBuilder;

// Roles.
const RID_TRADER = 'trader';
const RID_COMMITTEE = 'committee';


/**
 * Implements hook_form_alter().
 */
function cforge_form_alter(&$form, $form_state, $form_id) {
  switch ($form_id) {
    case 'user_login_form':
    case 'user_login_block':
      // Redirect the user to given destination or else the member front page
      $request = \Drupal::request();
      if (!$request->request->has('destination')) {
        $path = \Drupal::config('cforge.settings')->get('member_frontpage');
        $request->query->set('destination', $path);
      }
      break;
    case 'user_form':
    case 'user_register_form':
      // Css lays out the given_name and family_name fields better
      $form['#attached']['library'][] = 'cforge/cforge_css';
      break;
  }
}

/**
 * Implements hook_form_user_pass_alter().
 *
 * Because $form_state->setRedirect doesn't work while the form is being built.
 */
function cforge_form_user_pass_alter(&$form, $form_state) {
  \Drupal::request()->query->set('destination', '/user/login');
}

/**
 * Implements hook_block_view_BASE_BLOCK_ID_alter().
 *
 * Change the title of the user block.
 */
function cforge_block_view_system_menu_block_alter(&$build, $block) {
  if ($block->getPluginId() == 'system_menu_block:account') {
    $build['#configuration']['label'] = \Drupal::currentUser()->getDisplayName();
  }
}

/**
 * Implements hook_cron().
 */
function cforge_cron() {
  // Delete old sessions to keep the sessions table small.
  \Drupal::database()->delete('sessions')
    ->condition('uid', 0)
    ->condition('timestamp', strtotime('-1 day'), '<')
    ->execute();
  \Drupal::database()->delete('sessions')
    ->condition('timestamp', strtotime('-1 month'), '<')
    ->execute();
  // Keep the watchdog table small too.
  if (\Drupal::moduleHandler()->moduleExists('watchdog')) {
  \Drupal::database()->delete('watchdog')
      ->condition('timestamp', strtotime('-1 week'), '<')
      ->execute();
  }

  $conf = \Drupal::configFactory()->getEditable('cforge.settings');
  if ($conf->get('demo_mode')) {
    // Restore from backup, if we are in demo mode on the first cron of every day
    if (date('z', $conf->get('last_cron')) <> date('z', REQUEST_TIME)) {
      $info = Drupal\Core\Database\Database::getConnectionInfo();
      $db  = reset($info);
      $command = 'mysql -u '.$db['username'];
      if ($db['password']) {
        $command .= ' -p '.$db['password'];
      }
      $command .= ' '.$db['database'] .' < '.\Drupal::service('file_system')->realpath('private://dump.sql');
      exec($command);
      // This restoration hasn't deleted any files which were uploaded. We might use bash find for that.
    }
  }
  module_load_install('cforge');
  cforge_clear_old_files(\Drupal::service('app.root') .'/'. \Drupal::service('site.path').'/files/css');
  $conf->set('last_cron', REQUEST_TIME)->save();
}

/**
 * Implements hook_element_info_alter().
 *
 * This is all the code from the disablepwstrength module.
 */
function cforge_element_info_alter(&$types) {
  $processes = &$types['password_confirm']['#process'];
  if (isset($processes)) {
    $position = array_search('user_form_process_password_confirm', $processes);
    if ($position !== FALSE) {
      unset($processes[$position]);
    }
  }
}

/**
 * Implements hook_entity_info_alter().
 *
 * Add new Form handlers for the user profile edit forms.
 */
function cforge_entity_type_alter(array &$entity_types) {
  if (isset($entity_types['user'])) {
    $entity_types['user']->setFormClass('profile', "Drupal\user\ProfileForm");
    $entity_types['user']->setFormClass('admin', "Drupal\user\ProfileForm");
  }
}

/**
 * Implements hook_form_FORM_ID_alter().
 */
function cforge_form_user_form_alter(&$form, $form_state) {
  $form_object = $form_state->getFormObject();
  if (isset($form['phones'])) {
    // Remove the multiple field theme from the phone numbers.
    $form['phones']['widget'][0]['_weight']['#access'] = FALSE;
    $form['phones']['widget'][1]['_weight']['#access'] = FALSE;
    unset($form['phones']['widget']['#theme']);
  }
  if (isset($form['account'])) { // Should always be the case
    // @see cforge_entity_base_field_info_alter();
    unset($form['account']['current_pass']);
    $form['account']['status']['#options'][0] = t('User has not been approved or is no longer a member');
    $form['account']['status']['#options'][1] = t('User can log in normally');
    $form['account']['roles']['#options'][RID_COMMITTEE] .= ' - ' . (string) t('Can manage members, articles, pages images, & documents.');
    $form['account']['roles']['#options'][RID_TRADER] .= ' - ' . t('Can use the main currency.');
  }

  if (isset($form['address'])) {
    $form['address']['widget'][0]['#title'] = t('Name & address');
    // Unhide this which was hidden in Drupal\address\Plugin\Field\FieldWidget\AddressDefaultWidget::addressElements()
    unset($form['address']['widget'][0]['address_line2']['#title_display']);
    $form['#attached']['library'][] = 'cforge/cforge_css';
  }

  $user_entity = $form_object->getEntity();

  if ($form_object->getOperation() == 'default') { // The settings form display
    // Add the privacy settings fields and re-arrange the form.
    $form['privacy']  = [
      '#type' => 'details',
      '#title' => t('Privacy settings'),
      '#description' => t('Determine how much of your data other people can see. Committee members can see everything.'),
      '#open' => TRUE
    ];
    $form['privacy']['smallad_visibility'] = $form['smallad_visibility'];
    $form['privacy']['contact'] = $form['contact']['contact'];
    $form['privacy']['contact']['#title'] = t('Allow other members to email me');
    unset($form['smallad_visibility'],  $form['contact']);
    $user_data_service = \Drupal::service('user.data');

    // The checkboxes show TRUE for visible fields.
    $form['privacy']['privacy_phones'] = [
      '#title' => t('Show my phone number'),
      '#type' => 'checkbox',
      '#default_value' => !$user_data_service->get('cforge', $user_entity->id(), 'phones')
    ];
    $form['privacy']['privacy_address'] = [
      '#title' => t('Show my address'),
      '#type' => 'checkbox',
      '#default_value' => !$user_data_service->get('cforge', $user_entity->id(), 'address')
    ];
    $form['actions']['submit']['#submit'][] = 'cforge_user_privacy_settings_submit';
  }

  // Hide the 'present' checkbox for user 1, who is never present
  if (isset($form['present']) && $user_entity->id() == 1) {
    $form['present']['#disabled'] = TRUE;
    $form['present']['widget']['value']['#title'] .= ' ('. t('User 1 is never present').')';
  }
}

/**
 * Form submit callback
 */
function cforge_user_privacy_settings_submit($form, $form_state)  {
  $user_data_service = \Drupal::service('user.data');
  $user_entity_id = $form_state->getFormObject()->getEntity()->id();
  foreach (['privacy_phones' => 'phones', 'privacy_address' => 'address'] as $key => $val)  {
    // The service stores TRUE for private fields.
    if (!$form_state->getValue($key)) {
      $user_data_service->set('cforge', $user_entity_id, $val, TRUE);
    }
    else {
      $user_data_service->delete('cforge', $user_entity_id, $val);
    }
  }
  // Assume that the render cache will be cleared as part of the form submission.
}

/**
 * Implements hook field_widget_WIDGET_NAME_form_alter().
 *
 * Tailor the display of the phone multiple cardinality field.
 */
function cforge_field_widget_telephone_default_form_alter(&$element, $form_state, $context) {
  static $i = 0;
  $element['value']['#field_prefix']  = ($i % 2 == 1) ? t('Other phone:') : t('Mobile phone:');
  $i++;
}

/**
 * Implements hook_form_FORM_ID_alter().
 *
 * Tailor the account settings form.
 */
function cforge_form_user_admin_settings_alter(&$form) {
  $reg_fieldset = &$form['registration_cancellation'];
  unset(
    $reg_fieldset['#description'],
    $reg_fieldset['user_register']['#options'][UserInterface::REGISTER_VISITORS]
  );
  $reg_fieldset['#weight'] = -5;
  $reg_fieldset['user_register']['#title'] = t('How to add new members');
  $reg_fieldset['user_register']['#options'][UserInterface::REGISTER_ADMINISTRATORS_ONLY] = t('Only committee can create accounts');
  $reg_fieldset['user_register']['#options'][UserInterface::REGISTER_VISITORS_ADMINISTRATIVE_APPROVAL] = t('Anyone can create an account, then a committee member must enable it');
  $reg_fieldset['user_cancel_method']['#access'] = FALSE;
  $reg_fieldset['user_email_verification']['#access'] = FALSE;
  $form['contact']['#access'] = FALSE;
  $form['admin_role']['#access'] = FALSE;
}

/**
 * Implements hook_form_FORM_ID_alter().
 *
 * Other node types have no menus at all.
 */
function cforge_form_node_page_form_alter(&$form, &$form_state) {
  $home = \Drupal::config('system.site')->get('page.front');
  // Prevent changing the alias of the home page, and remove the delete button.
  if ($form['path']['widget'][0]['alias']['#default_value'] == $home) {
    $form['path']['warning'] = [
      '#markup' => t(
        "Warning: '@name' is your site's front page.",
        ['@name' => $home]
      ),
    ];
  }
}

/**
 * Implements hook_form_node_form_alter().
 *
 * Make the descriptions of the aliases more friendly.
 */
function cforge_form_node_form_alter(&$form, $form_state) {
  $node = $form_state->getFormObject()->getEntity();
  if ($node->bundle() == 'page') {
    $form['public'] = [
      '#title' => t('Visible to the public and admins only'),
      '#type' => 'checkbox',
      '#default_value' => cforge_node_get_anonaccess($node->id()),
    ];
    foreach (Element::children($form['actions']) as $name) {
      $actions[$name]['#submit'][] = 'cforge_node_submit';
    }
  }
  // Rename the url alias field and explain it better.
  $form['path']['widget'][0]['alias']['#title'] = t('Friendly address');
  $form['path']['widget'][0]['alias']['#description'] = t(
    "Optionally give your page a more meaningful address. E.g. 'articles-of-association'"
  );
}

/**
 * Implements hook_menu_links_discovered_alter().
 *
 * Remove unwanted menu items put in the main menu by core modules.
 */
function cforge_menu_links_discovered_alter(&$links) {
  unset(
    // Was disabled anyway.
    $links['filter.tips_all'],
    // Was disabled anyway.
    $links['search.view'],
    $links['views_view:views.smallads_directory.page_list']
  );
  $links['contact.site_page']['enabled'] = TRUE;
  // Having removed node.add_page its is helpful to rename the links under it to say 'add @type'
  if (\Drupal::entityTypeManager()->getDefinition('node_type', FALSE)) {// may not be installed yet.
    $node_types = \Drupal::entityTypeManager()->getStorage('node_type')->loadMultiple();
    foreach ($node_types as $type) {
      $type_id = $type->id();
      if (isset($links['node.add.' . $type_id])) {
        $links['node.add.' . $type_id]['title'] = t(
          'Add @type',
          ['@type' => $links['node.add.' . $type_id]['title']]
        );
      }
    }
  }
  if (isset($links['user.logout'])) {
    // This overrides admin_toolbar_tools_menu_links_discovered_alter()
    unset($links['user.logout']['parent']);
    $links['user.logout']['menu_name'] = 'account';
  }
  if (isset($links['mcapi.mass'])) {
    $links['mcapi.mass']['weight'] = 1;
  }
  if (isset($links['views_view:views.smallads_directory.collection'])) {
    $links['views_view:views.smallads_directory.collection']['menu_name'] = 'tools';
  }
}

/**
 * Implements hook_local_tasks_alter().
 */
function cforge_local_tasks_alter(&$definitions) {
  unset($definitions['smallads.admin']);//this refers to a view that this distro doesn't use.
  $definitions['entity.user.edit_form']['route_name'] = 'entity.user.profile';
}

/**
 * Implements hook_mail().
 */
function cforge_mail($key, &$message, $params) {
  if ($key == 'all') {
    $recipient = User::loadByProperties(['mail' => $message['to']]);
    // Tokens allowed were.
    $message['subject'] = $params['subject'];
    $message['body'][] = \Drupal::token()->replace($params['body'], ['user' => $recipient]);
  }
}

/**
 * Implements hook_mail_alter().
 *
 * Add the mail footer.
 */
function cforge_mail_alter(&$message) {
  if (!is_array($message['body'])) {
    // We assume it is a string.
    \Drupal::messenger()->addWarning('problem constructing body of mail '.$message['key']);
    $message['body'] = (array)$message['body'];
  }
  if (\Drupal::currentUser()->id()) {
    $message['body'][] = \Drupal::token()->replace(
      \Drupal::config('cforge.settings')->get('mail_footer'),
      $message['params'],
      [
        'language' => $message['langcode'],
        'sanitize' => FALSE,
      ]
    );
  }
  if (\Drupal::config('cforge.settings')->get('demo_mode')) {
    // Block the mail and print to screen.
    $message['send'] = FALSE;
    static $onlyone = FALSE;
    if (!$onlyone) {
      $subject = (string)t('Email intercepted: @subject', ['@subject' => $message['subject']]);
      \Drupal::messenger()->addStatus($subject ."\n\n". implode("\n\n", $message['body']));
      $onlyone = TRUE;
    }
  }
}

/**
 * Implements hook_form_system_site_information_settings_alter().
 */
function cforge_form_system_site_information_settings_alter(&$form, $form_state) {
  // Add a setting for the member frontpage
  $config = \Drupal::config('cforge.settings');
  $form['front_page']['member_frontpage'] = [
    '#title' => t("Member's front page"),
    '#description' => t("Users will be redirected to this page when they log in."),
    '#default_value' => $config->get('member_frontpage'),
    '#element_validate' => ['cforge_member_frontpage_validate'],
  ] + $form['front_page']['site_frontpage'];

  // Add the lat and lon fields
  $form['coordinates'] = [
    '#title' => t('Location'),
    '#description' => t(
      'To locate your community on a global map. Find yourself with @link',
      ['@link' => \Drupal::linkGenerator()->generate('latlong.net', \Drupal\Core\Url::fromUri('http://latlong.net'))]
    ),
    '#type' => 'details',
    '#open' => TRUE,
    '#weight' => 0,
    'lat' => [
      '#title' => t('Latitude'),
      '#type' => 'textfield',
      '#default_value' => $config->get('coordinates.lat'),
      '#element_validate' => ['element_validate_cf_is_numeric'],
      '#placeholder' => '0.0',
      '#required' => TRUE,
      '#weight' => 1,
    ],
    'lon' => [
      '#title' => t('Longitude'),
      '#type' => 'textfield',
      '#default_value' => $config->get('coordinates.lon'),
      '#element_validate' => ['element_validate_cf_is_numeric'],
      '#placeholder' => '0.0',
      '#required' => TRUE,
      '#weight' => 2,
    ]
  ];

  $form['#submit'][] = 'cforge_submit_system_site_information_settings';
}

/**
 * Element validation callback.
 */
function cforge_member_frontpage_validate(&$element, $form_state) {
  if ($element['#value'] && $element['#value'][0] !== '/') {
    $form_state->setError($element, t("The path '%path' has to start with a slash.", ['%path' => $element['#value']]));
  }
}

/**
 * Form submission callback.
 */
function cforge_submit_system_site_information_settings($form, $form_state) {
  \Drupal::configFactory()->getEditable('cforge.settings')
    ->set('member_frontpage', $form_state->getValue('member_frontpage'))
    ->set('coordinates.lat', $form_state->getValue('lat'))
    ->set('coordinates.lon', $form_state->getValue('lon'))
    ->save();
}

/**
 * Implements hook_locale_translation_projects_alter().
 *
 * To clear the locale_file table and on devel/php do
 *
 *  \Drupal::moduleHandler()->loadInclude('locale', 'fetch.inc');
 *  \Drupal::moduleHandler()->loadInclude('locale', 'bulk.inc');
 *  locale_translation_clear_status();
 *  $batch = locale_translation_batch_update_build(array(), $langcodes, array());
 *  batch_set($batch);
 *
 * @see locale_translation_check_projects_local()
 *
 * @todo this doesn't cover themes...
 */
function cforge_locale_translation_projects_alter(&$projects) {
  $modules = ['cforge', 'smallads', 'mutual_credit', 'community_tasks'];
  // Assumes all translations of a module are in the same place.
  foreach ($modules as $module) {
    // This applies to all languages.
    $projects[$module]['server_pattern'] = 'http://translations.communityforge.net/%core/%project.%language.po';
  }
}

/**
 * Implements hook_form_system_modules_alter().
 *
 * Add Readme links to the modules page, where readme files exist.
 */
function cforge_form_system_modules_alter(&$form, &$form_state) {
  if (\Drupal::moduleHandler()->moduleExists('cforge_import')) {
    \Drupal::messenger()->addWarning(t('Do uninstall the Cforge Import module when you have finished with it.'));
  }
  // Link to the readme file. (cool idea taken from the advanced help module).
  foreach (Element::children($form['modules']) as $group) {
    foreach (Element::children($form['modules'][$group]) as $module) {
      $path = DRUPAL_ROOT . '/' . drupal_get_path('module', $module);
      if (file_exists($path . '/README.txt')) {
        $form['modules'][$group][$module]['links']['help'] = [
          '#type' => 'link',
          '#title' => t('README.txt'),
          '#url' => Url::fromRoute('cforge.readme', ['module' => $module]),
          '#attributes' => [
            'class' => ['module-link'],
          ],
        ];
      }
    }
  }
}

/**
 * Implements hook_system_info_alter().
 *
 * Move things around on the modules page.
 */
function cforge_system_info_alter(&$info, Extension $extension, $type) {
  // Make some contrib modules into cforge features!
  switch ($extension->getName()) {
    case 'community_tasks':
      $info['package'] = 'Cforge features';
  }
}

/**
 * Implements hook_form_alter().
 *
 * Clears the css cache after a modification.
 *
 * @todo test when css_injector is available
 */
function cforge_form_css_injector_edit_alter(&$form, &$form_state) {
  $form['buttons']['save']['#submit'][] = 'drupal_clear_css_cache';
  $form['buttons']['save_and_continue']['#submit'][] = 'drupal_clear_css_cache';
}

/**
 * Utility function to make batch mail a little easier for cforge dependencies.
 *
 * @param string $module
 *   The name of the modue which is responsible for the mail.
 * @param string $key
 *   The key of that mail.
 * @param array $uids
 *   The Uids of the recipients.
 * @param array $params
 *   The mail params.
 * @param \Drupal\Core\Url
 *   Where to redirect after tha batch is run.
 * @param string $finished
 *   The path to go to at the end of the batch.
 *
 * @todo test batch mail in d8.
 */
function cforge_batch_mail($module, $key, array $uids, array $params, $finished = '') {
  if (empty($uids)) {
    return;
  }
  $batch_builder = (new BatchBuilder())
    ->setTitle(t('Sending @count mails', ['@count' => count($uids)]))
    ->setProgressMessage(t('Processed @current out of @total.'));

  foreach ($uids as $uid) {
    $batch_builder->addOperation('cforge_batch_mail_send', [$module, $key, $uid, $params]);
  }

  if ($finished) {
    $batch_builder->setFinishCallback($finished);
  }

  batch_set($batch);
}

/**
 * Batch callback.
 */
function cforge_batch_mail_send($module, $key, $uid, $params, $progress) {
  $account = user_load($uid);
  $params['user'] = $account;
  \Drupal::service('plugin.manager.mail')->mail(
    $module,
    $key,
    $account->getEmail(),
    $account->getPreferredLangcode(),
    $params
  );
}

/**
 * Implements hook_entity_type_view().
 */
function cforge_mcapi_transaction_view(array $build) {
  //mdump($build);die();
}


/**
 * Implements hook_node_access().
 */
function cforge_node_access($node, $op, $account) {
  $result = AccessResult::neutral();
  if ($op == 'view') {
    // Handle the public-only nodes.
    $public = cforge_node_get_anonaccess($node->id());
    if ($node->bundle() == 'page') {
      $result = AccessResult::allowedIf($public == $account->isAnonymous());
    }
    else {
      $result = AccessResult::allowedIf($public || $account->isAuthenticated());
    }
  }
  return $result->cachePerPermissions();
}

/**
 * Implements hook_form_taxonomy_term_categories_form_alter().
 *
 * Force the term description field to be plain text. There doesn't seem to be a
 * way of doing this by altering the baseFieldDefinition.
 */
function cforge_form_taxonomy_term_categories_form_alter(&$form, $form_state) {
  $form['description']['widget'][0]['format']['#allowed_formats'] = ['plain_text'];
}

/**
 * Form submission handler.
 */
function cforge_node_submit($form, $form_state) {
  cforge_node_set_anonaccess($form_state->getFormObject()->getEntity()->id(), $form_state->getValue('publicevent'));
}

/**
 * Implements hook_node_delete().
 */
function cforge_node_delete($node) {
  \Drupal::keyValue('anonaccess')->delete($node->id);
}

/**
 * Determine if a node is intended for anon.
 *
 * @param int $nid
 */
function cforge_node_get_anonaccess($nid) {
  return \Drupal::keyValue('anonaccess')->get($nid);
}

/**
 * Form submit callback.
 *
 * Set the anonaccess flag for a node.
 */
function cforge_node_set_anonaccess($nid, $value) {
  \Drupal::keyValue('anonaccess')
    ->set($nid, $value);
}


/**
 * Determine whether the referral system is active according to cforge settings.
 *
 * @return bool
 *   TRUE if the referral fee is not 0 and there is a source wallet.
 */
function cforge_referral() {
  $conf = \Drupal::Config('cforge.settings');
  return !empty_worths_config($conf->get('referral_fee')) && $conf->get('referral_src');
}

/**
 * Implements hook_ENTITY_TYPE_insert().
 *
 * Modify default content view to remove unwanted bulk actions. Unfortunately I
 * can't see how to do this while installing. since the view hasn't been created
 * by the time of cforge_install
 */
function cforge_view_insert($view) {
  if ($view->id() == 'content') {
    $display = &$view->getDisplay('default');
    $display['display_options']['fields']['node_bulk_form']['include_exclude'] = 'include';
    $display['display_options']['fields']['node_bulk_form']['selected_actions'] = [
      'node_delete_action',
      'node_publish_action',
      'node_unpublish_action',
    ];
    $view->save();
  }
}

/**
 * Implements hook_user_presave().
 */
function cforge_user_presave($account) {
  // If the user is being created, add the trader role.
  if ($account->isNew()) {
    $account->addRole(RID_TRADER);
  }
  if ($account->id() == 1) {
    // Hide user 1 from offers/want listings
    //TODO should hide this field on the user 1 form
    $account->present->value = 0;
  }
}

/**
 * Implements hook_entity_base_field_info_alter().
 *
 * Remove the protected fields from the user and hence the need to confirm with
 * a password on the user edit form.
 */
function cforge_entity_base_field_info_alter(&$fields, $entity_type) {
  // Alter the mymodule_text field to use a custom class.
  if ($entity_type->id() == 'user') {
    foreach (['mail', 'pass'] as $fieldname) {
      $constraints = $fields[$fieldname]->getConstraints();
      unset($constraints['ProtectedUserField']);
      $fields[$fieldname]->setConstraints($constraints);
    }
  }
}

/**
 * Implements hook_migrate_prepare_row().
 */
function cforge_migrate_prepare_row($row, $source, $migration) {
  module_load_include('migrate.inc', 'cforge');
  return _cforge_migrate_prepare_row($row, $source, $migration);
}

 /**
  * Implements hook_migration_plugins_alter().
  */
function cforge_migration_plugins_alter(array &$definitions)  {
  module_load_include('migrate.inc', 'cforge');
  _cforge_migration_plugins_alter($definitions);
}

 /**
  * Implements hook_migrate_field_info_alter().
  */
function cforge_migrate_field_info_alter(array &$fields) {
  $fields['list']['type_map']['list_float'] = 'list_float';
}

/**
 * Implements hook_module_preinstall().
 *
 * Prepare the content directory for the right language.
 * The default_content module fails badly if the serialised content contains
 * language versions unknown to the site, it throws "Invalid translation
 * language (fr) specified" So ATM the content is prepared in single languge
 * nodes, to be imported one language per site.
 *
 * @temp until the default_content module copes with content in languages not
 * enabled on the site. When removing see also cforge_modules_installed().
 */
function cforge_module_preinstall($module) {
  $content_path = DRUPAL_ROOT . '/'.drupal_get_path('module', $module).'/content';
  @unlink($content_path);
  if (file_exists($content_path. '-en')) {
    $content_source = $content_path . '-' . \Drupal::languageManager()->getDefaultLanguage()->getId();
    if (!file_exists($content_source)) {
      $content_source = $content_path.'-en';
    }
    if (symlink($content_source, $content_path)) {
      \Drupal::logger('Default Content')->debug(
        'Created symlink from @source to @dest.',
        ['@source' => $content_source, '@dest' => $content_path]
      );
    }
    else {
      \Drupal::logger('Default Content')->warning(
        'Unable to create symlink from @source to @dest.',
        ['@source' => $content_source, '@dest' => $content_path]
      );
    }
  }
}

/**
 * Implements hook_modules_installed().
 */
function cforge_modules_installed($modules) {
  if (in_array('personal_digest', $modules)) {
    if ($conf = EntityFormDisplay::load('user.user.default')) {
      // Goes to the bottom.
      $conf->setComponent('personal_digest')->save();
    }
    if ($conf = EntityFormDisplay::load('user.user.profile')) {
      $conf->removeComponent('personal_digest')->save();
    }
  }
  if (in_array('community_tasks', $modules)) {
    Role::load('trader')
      ->grantPermission('commit to tasks')
      ->save();
    // These roles need to be handled in the upgrade
    Role::load('committee')
      ->grantPermission('create community_task content')
      ->grantPermission('edit any community_task content')
      ->grantPermission('delete any community_task content')
      ->save();
  }
  // @temp until default_content handles multiple langauges
  foreach($modules as $module) {
    $path = DRUPAL_ROOT . '/'.drupal_get_path('module', $module).'/content';
    if (file_exists($path)) {
      unlink($path);
    }
  }
}

/**
 * Implements hook_modules_uninstalled().
 */
function cforge_modules_uninstalled($modules) {
  if (in_array('migrate', $modules)) {
    // Delete all the tables beginning with migrate.
    foreach (\Drupal::database()->query("show tables like 'migrate_%'")->fetchCol() as $table) {
      \Drupal::database()->schema()->dropTable($table);
    }
  }
}

/**
 * Implements hook_migrate_destination_info_alter().
 *
 * @temp until something changes in core coz this always fails
 */
function cforge_migrate_destination_info_alter(&$plugins) {
  $plugins['null']['requirements_met'] = TRUE;
}

/**
 * Implements hook_field_access().
 */
function cforge_entity_field_access($operation, $field_definition, $account, $items = NULL) {
  $fname = $field_definition->getName();
  $result = AccessResult::neutral();
  if($fname == 'notes_admin') {
    // This user field can only be viewed or operated on by committee
    $result = AccessResult::forbiddenIf(!$account->hasPermission('administer users'), 'Visible to committee only');
  }
  elseif ($operation == 'view' && $account->isAnonymous()){
    $result = AccessResult::ForbiddenIf(in_array($fname, ['user_picture', 'uid']), 'Anon must never see user info');
  }
  elseif($operation == 'view' && !$account->hasPermission('administer users')) {
    if ($fname == 'phones' or $fname == 'address') {
      $uid = $items->getEntity()->id();
      if ($uid != $account->id()) {
        // We know the entity type is a user
        $private = \Drupal::service('user.data')->get('cforge', $uid, $fname);
        $result = AccessResult::ForbiddenIf($private, 'Forbidden by user preferences');
      }
    }
  }
  return $result->cachePerPermissions();
}

/**
 * Utility
 *
 * When building the block admin form, prevent changing the
 * region or weight.
 */
function cforge_fix_block(&$form, $form_state, $id, $region_key, $weight) {
  $aggBlock = &$form['blocks'][$id];
  if (!$aggBlock) {dsm(array_keys($form['blocks']), $id);return;}
  $aggBlock['region-theme']['region']['#options'] = array_intersect_key(
    $aggBlock['region-theme']['region']['#options'],
    [$region_key => 1]
  );
  $aggBlock['operations']['#links'] = array_intersect_key(
    $aggBlock['operations']['#links'],
    ['devel' => 1, 'translate' => 1]
  );
  //unset($aggBlock['operations']['#links']);
  array_unshift($form['#submit'], 'cforge_fix_block_weight');
  $weights = $form_state->get('fixblock', []);
  $weights[$id] = $weight;
  $form_state->set('fixblock', $weights);
}

/**
 * Form submit callback.
 * Fix the weight of a block when the blocks admin form is saved.
 */
function cforge_fix_block_weight($form, $form_state) {
  foreach ($form_state->get('fixblock', []) as $block_id => $weight) {
    $form_state->setValue(['blocks', $block_id, 'weight'], $weight);
  }
}

/**
 * Implements hook_block_view_BLOCK_ID_alter().
 *
 * Rename the aggregator blocks with the names of the feed.
 *
 * @note Waiting a setting in the aggregator block to do do this
 */
function cforge_block_view_aggregator_feed_block_alter(&$build, $block) {
  if ($feed = Feed::load($build['#configuration']['feed'])) {
    $build['#configuration']['label'] = $feed->label();
    $build['#configuration']['label_display'] = 'visible';
  }
}

/**
 * Element validate callback
 */
function element_validate_cf_is_numeric(&$element, $form_state) {
  if (!is_numeric($element['#value'])) {
    $form_state->setError($element, t('Must be a number.'));
  }
}

function cforge_form_contact_message_personal_form_alter(&$form, $form_state) {
  $params = \Drupal::request()->query->all();
  $form['subject']['widget'][0]['value']['#default_value'] = $params['subject'];
  dsm($form['subject']);
}
