<?php

namespace Drupal\cforge\EventSubscriber;

use Drupal\address\Event\AddressFormatEvent;
use Drupal\address\Event\AddressEvents;
use Drupal\Core\Logger\LoggerChannel;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\HttpKernel\Event\FilterResponseEvent;
use Symfony\Component\HttpKernel\Event\GetResponseEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Cforge react to events.
 */
class CforgeSubscriber implements EventSubscriberInterface {

  private $currentUser;
  private $cforgeSettings;
  private $pathmatcher;
  private $logger;

  function __construct($current_user, $config, $path_matcher, LoggerChannel $logger_channel) {
    $this->currentUser = $current_user;
    $this->cforgeSettings = $config->get('cforge.settings');
    $this->pathMatcher = $path_matcher;
    $this->logger = $logger_channel;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events = [
      KernelEvents::REQUEST  => [['cforgeBlockBot', -1001]],
      KernelEvents::RESPONSE  => [['cforgeFront', 1001]]
    ];
    if (class_exists('Drupal\address\Event\AddressEvents')) {
      $events[AddressEvents::ADDRESS_FORMAT] = [['rewriteAddress']];
    }
    return $events;
  }

  /**
   * Redirect users
   */
  public function cforgeFront(FilterResponseEvent $event) {
    // Don't run this when site is in maintenance mode.
    if (\Drupal::state()->get('system.maintenance_mode')) {
      return;
    }
    // Ignore non index.php requests (like cron).
    if (!empty($_SERVER['SCRIPT_FILENAME']) && realpath(DRUPAL_ROOT . '/index.php') != realpath($_SERVER['SCRIPT_FILENAME'])) {
      return;
    }
    if ($this->currentUser->isAuthenticated()) {
      $path = trim($event->getRequest()->getpathinfo(), '/');
      if (empty($path)) {
        $front = $this->cforgeSettings->get('member_frontpage');
        $event->setResponse(new RedirectResponse($front));
      }
    }
  }

  public function cforgeBlockBot(GetResponseEvent $event) {
    // Ignore non index.php requests (like cron).
    if (!empty($_SERVER['SCRIPT_FILENAME']) && realpath(DRUPAL_ROOT . '/index.php') != realpath($_SERVER['SCRIPT_FILENAME'])) {
      return;
    }
    if ($this->currentUser->isAnonymous()) {
      // This was originally added to deter certain kinds of bots probing long paths
      $path = trim($event->getRequest()->getpathinfo(), '/');
      $parts = array_filter(explode('/', $path));
      if (count($parts) > 5) {
        $response = new RedirectResponse('Access denied', 404);
        $event->setResponse($response);
        $this->logger->notice("Blocked access to path %path", ['%path' => $path]);
      }
    }
  }

  /**
   * Ensure every address format has %dependentLocality, sticking it on the end
   * if necessary, that that field is required, and called neighbourhood
   */
  public function rewriteAddress(AddressFormatEvent $event) {
    $addressFormat = $event->getDefinition();
    $pattern = $addressFormat['format'];
    if (!strpos($pattern, '%dependentLocality')) {
      $pattern .= "\n%dependentLocality";
      $addressFormat['format'] = $pattern;
    }
    $addressFormat['dependent_locality_type'] = 'neighborhood';
    $addressFormat['required_fields'] = ['dependentLocality', 'givenName'];
    $event->setDefinition($addressFormat);
  }

}
