<?php

namespace Drupal\cforge\Installer;

use Drupal\node\Entity\Node;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Config\InstallStorage;
use Drupal\Core\Extension\ModuleInstallerInterface;
use Drupal\Core\Session\AccountSwitcherInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Utility\Token;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Choose between different Cforge flavours.
 */
class ChooseFlavour extends FormBase {

  /**
   * @var ModuleInstallerInterface
   */
  private  $installer;

  /**
   * @var AccountSwitcherInterfacev
   */
  private $accountSwitcher;

  /**
   * @var LanguageManagerInterface
   */
  private $languageManager;

  /**
   * @var Token
   */
  private $token;

  /**
   * @var string
   */
  private $root;

  /**
   * @param string $root
   * @param ModuleInstallerInterface $module_installer
   * @param AccountSwitcherInterface $account_switcher
   * @param LanguageManagerInterface $language_manager
   * @param Token $token
   */
  public function __construct($root, ModuleInstallerInterface $module_installer, AccountSwitcherInterface$account_switcher, LanguageManagerInterface $language_manager, Token $token) {
    $this->root = $root;
    $this->installer = $module_installer;
    $this->accountSwitcher = $account_switcher;
    $this->languageManager = $language_manager;
    $this->token = $token;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('app.root'),
      $container->get('module_installer'),
      $container->get('account_switcher'),
      $container->get('language_manager'),
      $container->get('token')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'cforge_flavour_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form_state->loadInclude('hamlets', 'module');
    $form['#title'] = t('Choose flavour');
    module_load_install('cforge');
    $options = cforge_available_flavours();
    asort($options);
    $form['flavour'] = [
      '#title' => $this->t('Flavour', [], ['context' => 'Installation variety']),
      '#type' => 'radios',
      '#options' => $options,
      '#default_value' => 'hamlets',
      '#required' => TRUE,
      '#weight' => 4,
    ];
    $form['upgrade'] = [
      '#title' => t('Upgrade from Drupal 7'),
      '#type' => 'checkbox',
      '#weight'=> 8
    ];

    $form['devel'] = [
      '#title' => t('Devel modules'),
      '#type' => 'checkbox',
      '#weight'=> 9
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => 'submit',
      '#weight' => 10,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $flavour_module = $form_state->getValue('flavour');
    module_set_weight($flavour_module, 100);
    $modules[] = $flavour_module;

    if ($form_state->getValue('upgrade')) {
      $modules = array_merge($modules, [
        'migrate',
        'migrate_drupal',
        'migrate_drupal_ui',
        'migrate_plus',
        'migrate_tools',
        'migrate_upgrade',
        'options'
      ]);
    }
    if ($form_state->getValue('devel')) {
      $modules = array_merge($modules, [
        'devel',
        'dblog',
        'field_ui',
        'views_ui',
        'contextual',
        'admin_toolbar_tools',
        'config',
        'config_translation'
      ]);
    }

    foreach (system_rebuild_module_data() as $module_name => $extension) {
      $info = $extension->info;
      // Modules specific to Cforge hosting.
      if ($info['package'] == 'Cforge features' and !empty($info['hidden'])) {
        $modules[] = $module_name;
      }
    }

    $this->clearPreexistingConfig($flavour_module);

    // The default content module is already installed and will pick up from the
    // content directory in each module installed here
    $mod_names = implode(', ', $modules);
    if (!$this->installer->install($modules)) {
      throw new \Exception('Failed to install all modules '.$mod_names);
    }

    // Swop out the tokens in the privacy policy template and tell the cookie
    // compliance module what path it is.
    if ($node = Node::load(NID_PRIVACY_POLICY)) {
      $node->body->value = $this->token->replace($node->body->value, [], ['sanitize' => TRUE]);
      $node->save();
    }
    else {
      $this->messenger()->addWarning('Unable to find privacy node to replace tokens');
    }

    // Assume all the default content is public-only except the privacy policy (node 1);
    // Set imported nodes to public using our 'public' flag for that purpose.
    foreach (Node::loadMultiple() as $node) {
      if ($node->bundle() == 'page' and $node->id() > 1) {
        cforge_node_set_anonaccess($node->id(), TRUE);
      }
    }
  }

  /**
   * Before installing a flavour remove any duplicate config.
   *
   * @param string $module
   */
  function clearPreexistingConfig($module) {
    $dir = drupal_get_path('module', $module) . '/' . InstallStorage::CONFIG_INSTALL_DIRECTORY;
    foreach (file_scan_directory($dir, '/.*yml/') as $path => $file) {
      if ($conf = $this->configFactory()->getEditable($file->name)) {
        if ($conf->getRawData()) {
          $conf->delete();
        }
      }
    }
  }

}
